@@ -53,6 +53,10 @@ class Client(object):
     .. note::
         Parameters like ``include_children`` and ``recurse`` are not used
         when paths contain globs.
+
+    .. note::
+        Different Hadoop distributions use different protocol versions. Snakebite defaults to 7, but this can be set by passing
+        in the ``hadoop_version`` parameter to the constructor.
     '''
     FILETYPES = {
         1: "d",
@@ -60,17 +64,19 @@ class Client(object):
         3: "s"
     }
 
-    def __init__(self, host, port):
+    def __init__(self, host, port, hadoop_version=7):
         '''
         :param host: Hostname or IP address of the NameNode
         :type host: string
         :param port: RPC Port of the NameNode
         :type port: int
+        :param hadoop_version: What hadoop protocol version should be used (default: 7)
+        :type hadoop_version: int
         '''
         self.host = host
         self.port = port
         self.service_stub_class = client_proto.ClientNamenodeProtocol_Stub
-        self.service = RpcService(self.service_stub_class, self.port, self.host)
+        self.service = RpcService(self.service_stub_class, self.port, self.host, hadoop_version)
 
     def ls(self, paths, recurse=False, include_toplevel=False, include_children=True):
         ''' Issues 'ls' command and returns a list of maps that contain fileinfo
